# Generated by Django 4.0.3 on 2022-03-29 02:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reviewpanel', '0004_cohort_size'),
    ]

    operations = [
        migrations.AddField(
            model_name='presentation',
            name='options',
            field=models.JSONField(blank=True, default=dict),
        ),
    ]
