{% extends "admin/change_form.html" %}

{% block extrastyle %}
  {{ block.super }}
  
  <style>
    p.paginator .btn-page {
      font-size: initial;
      cursor: initial;
    }
    p.paginator a.btn-page {
      cursor: pointer;
    }
  </style>
{% endblock %}
