
div#content {
  top: 80px;
}

div.section {
  position: absolute;
  padding-bottom: 16px;
}

div.section div.wrapper {
  position: relative;
  width: 100%;
}

div.section div.wrapper div.content {
  position: absolute;
  top: 0; left: 0; right: 0;
  display: flex;
  flex-direction: column;
}

div.block-reference,div.item-reference {
  margin-bottom: 8px;
}

div.item {
  width: 100%;
  height: 100%;
}

img.item_image,video.item_video {
  width: 100%;
  height: 100%;
}

img.item_image { object-fit: scale-down; }
video.item_video { object-fit: contain; }

div.item_video {
  background-color: black;
}

div.item_audio {
  display: flex;
  justify-content: center;
  align-items: center;
}

audio.item_audio {
  width: 75%;
}

object.item_document {
  object-fit: fill;
  width: 100%;
  height: 100%;
}

.item-selector-section {
  position: relative;
}

div.item-selector {
  width: 100%;
  white-space: nowrap;
  scroll-behavior: smooth;
  overflow-x: auto;
  overflow-y: hidden;
  -webkit-overflow-scrolling: touch;
  scroll-snap-type: x mandatory;
  scroll-padding: 0 10px;
}

div.item-thumb-container {
  width: min-content;
}

div.item-thumbnail {
  display: inline-block;
  width: 150px;
  scroll-snap-align: start;
  scroll-snap-stop: always;
}

div.item-thumbnail img {
  width: 100%;
  height: 100px;
  object-fit: contain;
}

div.item-thumbnail .item-label {
  width: fit-content;
  margin: 0 auto;
  height: 53px;
  line-height: 53px;
  cursor: default;
}

button.thumb-advance {
  position: absolute;
  top: 0;
  bottom: 0;
  opacity: 0;
  transition: opacity .3s;
  padding: 0;
  width: 28px;
}
.thumb-advance-left {
  left: 0;
  text-align: left;
  padding-right: 8px;
  background: linear-gradient(to right, rgba(255,255,255,1) 0%,
                                        rgba(255,255,255,1) 30%,
                                        rgba(255,255,255,0) 100%);
}
.thumb-advance-right {
  right: 0;
  text-align: right;
  padding-left: 8px;
  background: linear-gradient(to left, rgba(255,255,255,1) 0%,
                                       rgba(255,255,255,1) 30%,
                                       rgba(255,255,255,0) 100%);
}
svg.advancer-icon {
  cursor: pointer;
  padding: 8px 0;
  width: 20px;
}
.thumb-advance-left svg.advancer-icon { padding-right: 8px; }
.thumb-advance-right svg.advancer-icon { padding-left: 8px; }

div.input-score {
  display: flex;
  justify-content: center;
  align-items: center;
}

div.input-score * {
  flex: 1;
}

div.input-score input[type="text"],input[type="number"] {
  height: 24px;
}

div.input-score input:not([type="submit"]) {
  width: 50px;
}

div.input-score input[type="text"] {
  width: 95%;
}

div.input-score:not(:last-child) {
  margin-bottom: 8px;
}

div#modal {
  display: none;
  position: fixed;
  z-index: 1;
  left: 0;
  top: 0;
  width: 100%;
  height: 100%;
  background-color: rgba(0,0,0,0.9);
}

.modal-content {
  margin: auto;
  display: block;
  width: 90%;
  height: 100%;
  object-fit: scale-down;
}

div#top {
  top: 25px;
  height: 0;
}

div#topleft {
  position: absolute;
  top: 0;
  left: 0;
}

div#topcenter {
  width: 100%;
  text-align: center;
}

div#topright {
  position: absolute;
  top: 0;
  right: 0;
}
