
body {
  font: 100% sans-serif;
  padding: 0 16px;
}

div.center-fixed {
  margin: 80px auto 20px;
  width: 100%;
  max-width: 600px;
}

table.index {
  border-collapse: collapse;
  box-shadow: 0 0 20px rgba(0, 0, 0, 0.15);
  width: 100%;
  margin-bottom: 2em;
}

table.index tbody tr {
  border-bottom: thin solid #dddddd;
}

table.index th, td {
  padding: 12px 16px;
}

.table-controls {
  width: 100%;
  position: relative;
}

.table-controls > div {
  position: absolute;
  right: 0;
  bottom: 8px;
}
