# Generated by Django 4.0.4 on 2022-04-22 17:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reviewpanel', '0006_alter_cohortmember_object_id_alter_score_object_id_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='metric',
            name='display_values',
            field=models.BooleanField(default=False),
        ),
    ]
