# Generated by Django 4.0.3 on 2022-03-26 22:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reviewpanel', '0003_alter_cohort_inputs_alter_panel_panelists_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='cohort',
            name='size',
            field=models.PositiveIntegerField(default=0, editable=False),
        ),
    ]
